# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncSinglePage, AsyncSinglePage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.zero_trust.dlp.data_tag_categories import data_tag_create_params, data_tag_update_params
from .....types.zero_trust.dlp.data_tag_categories.data_tag_get_response import DataTagGetResponse
from .....types.zero_trust.dlp.data_tag_categories.data_tag_list_response import DataTagListResponse
from .....types.zero_trust.dlp.data_tag_categories.data_tag_create_response import DataTagCreateResponse
from .....types.zero_trust.dlp.data_tag_categories.data_tag_update_response import DataTagUpdateResponse

__all__ = ["DataTagsResource", "AsyncDataTagsResource"]


class DataTagsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DataTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DataTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DataTagsResourceWithStreamingResponse(self)

    def create(
        self,
        category_id: str,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCreateResponse]:
        """
        Creates a data tag in a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags",
                account_id=account_id,
                category_id=category_id,
            ),
            body=maybe_transform(
                {
                    "name": name,
                    "description": description,
                },
                data_tag_create_params.DataTagCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCreateResponse]], ResultWrapper[DataTagCreateResponse]),
        )

    def update(
        self,
        tag_id: str,
        *,
        account_id: str,
        category_id: str,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagUpdateResponse]:
        """
        Updates a data tag in a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags/{tag_id}",
                account_id=account_id,
                category_id=category_id,
                tag_id=tag_id,
            ),
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                data_tag_update_params.DataTagUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagUpdateResponse]], ResultWrapper[DataTagUpdateResponse]),
        )

    def list(
        self,
        category_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DataTagListResponse]:
        """
        Lists data tags in a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags",
                account_id=account_id,
                category_id=category_id,
            ),
            page=SyncSinglePage[DataTagListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DataTagListResponse,
        )

    def delete(
        self,
        tag_id: str,
        *,
        account_id: str,
        category_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a data tag from a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags/{tag_id}",
                account_id=account_id,
                category_id=category_id,
                tag_id=tag_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        tag_id: str,
        *,
        account_id: str,
        category_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagGetResponse]:
        """
        Gets a data tag from a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags/{tag_id}",
                account_id=account_id,
                category_id=category_id,
                tag_id=tag_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagGetResponse]], ResultWrapper[DataTagGetResponse]),
        )


class AsyncDataTagsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDataTagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataTagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataTagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDataTagsResourceWithStreamingResponse(self)

    async def create(
        self,
        category_id: str,
        *,
        account_id: str,
        name: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagCreateResponse]:
        """
        Creates a data tag in a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags",
                account_id=account_id,
                category_id=category_id,
            ),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "description": description,
                },
                data_tag_create_params.DataTagCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagCreateResponse]], ResultWrapper[DataTagCreateResponse]),
        )

    async def update(
        self,
        tag_id: str,
        *,
        account_id: str,
        category_id: str,
        description: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagUpdateResponse]:
        """
        Updates a data tag in a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags/{tag_id}",
                account_id=account_id,
                category_id=category_id,
                tag_id=tag_id,
            ),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                },
                data_tag_update_params.DataTagUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagUpdateResponse]], ResultWrapper[DataTagUpdateResponse]),
        )

    def list(
        self,
        category_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DataTagListResponse, AsyncSinglePage[DataTagListResponse]]:
        """
        Lists data tags in a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags",
                account_id=account_id,
                category_id=category_id,
            ),
            page=AsyncSinglePage[DataTagListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=DataTagListResponse,
        )

    async def delete(
        self,
        tag_id: str,
        *,
        account_id: str,
        category_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a data tag from a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags/{tag_id}",
                account_id=account_id,
                category_id=category_id,
                tag_id=tag_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        tag_id: str,
        *,
        account_id: str,
        category_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DataTagGetResponse]:
        """
        Gets a data tag from a category.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/data_tag_categories/{category_id}/data_tags/{tag_id}",
                account_id=account_id,
                category_id=category_id,
                tag_id=tag_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DataTagGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DataTagGetResponse]], ResultWrapper[DataTagGetResponse]),
        )


class DataTagsResourceWithRawResponse:
    def __init__(self, data_tags: DataTagsResource) -> None:
        self._data_tags = data_tags

        self.create = to_raw_response_wrapper(
            data_tags.create,
        )
        self.update = to_raw_response_wrapper(
            data_tags.update,
        )
        self.list = to_raw_response_wrapper(
            data_tags.list,
        )
        self.delete = to_raw_response_wrapper(
            data_tags.delete,
        )
        self.get = to_raw_response_wrapper(
            data_tags.get,
        )


class AsyncDataTagsResourceWithRawResponse:
    def __init__(self, data_tags: AsyncDataTagsResource) -> None:
        self._data_tags = data_tags

        self.create = async_to_raw_response_wrapper(
            data_tags.create,
        )
        self.update = async_to_raw_response_wrapper(
            data_tags.update,
        )
        self.list = async_to_raw_response_wrapper(
            data_tags.list,
        )
        self.delete = async_to_raw_response_wrapper(
            data_tags.delete,
        )
        self.get = async_to_raw_response_wrapper(
            data_tags.get,
        )


class DataTagsResourceWithStreamingResponse:
    def __init__(self, data_tags: DataTagsResource) -> None:
        self._data_tags = data_tags

        self.create = to_streamed_response_wrapper(
            data_tags.create,
        )
        self.update = to_streamed_response_wrapper(
            data_tags.update,
        )
        self.list = to_streamed_response_wrapper(
            data_tags.list,
        )
        self.delete = to_streamed_response_wrapper(
            data_tags.delete,
        )
        self.get = to_streamed_response_wrapper(
            data_tags.get,
        )


class AsyncDataTagsResourceWithStreamingResponse:
    def __init__(self, data_tags: AsyncDataTagsResource) -> None:
        self._data_tags = data_tags

        self.create = async_to_streamed_response_wrapper(
            data_tags.create,
        )
        self.update = async_to_streamed_response_wrapper(
            data_tags.update,
        )
        self.list = async_to_streamed_response_wrapper(
            data_tags.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            data_tags.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            data_tags.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .token_create_params import TokenCreateParams as TokenCreateParams
from .token_list_response import TokenListResponse as TokenListResponse
from .token_create_response import TokenCreateResponse as TokenCreateResponse
from .token_delete_response import TokenDeleteResponse as TokenDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["ContentExportParams", "DLPProfileInformation", "DLPProfileInformationEntry", "Order"]


class ContentExportParams(TypedDict, total=False):
    account_id: Required[str]

    dlp_profile_information: Required[Iterable[DLPProfileInformation]]
    """DLP profile metadata for the export."""

    dlp_profile_id: SequenceNotStr[str]
    """Filter by DLP profile IDs."""

    integration_id: SequenceNotStr[str]
    """Filter by integration IDs."""

    max_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view content flagged on or before this date."""

    min_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view content flagged on or after this date."""

    orders: Iterable[Order]
    """Ordering specifications for the export."""

    search: str
    """Search term to filter content."""

    vendors: List[
        Literal[
            "ANTHROPIC",
            "AWS",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SERVICENOW",
            "SLACK",
        ]
    ]
    """Filter by vendor types."""


class DLPProfileInformationEntry(TypedDict, total=False):
    """Entry within a DLP profile."""

    id: Required[str]
    """Unique identifier for the DLP profile entry."""

    name: Required[str]
    """Name of the DLP profile entry."""

    profile_id: Required[str]
    """ID of the parent DLP profile."""


class DLPProfileInformation(TypedDict, total=False):
    """DLP profile configuration."""

    id: Required[str]
    """Unique identifier for the DLP profile."""

    entries: Required[Iterable[DLPProfileInformationEntry]]
    """Entries contained within this DLP profile."""

    name: Required[str]
    """Name of the DLP profile."""


class Order(TypedDict, total=False):
    """Generic ordering specification."""

    direction: Required[Literal["asc", "desc"]]
    """Sort direction."""

    name: Required[Literal["asset_name", "dlp_profile_count", "integration_name", "latest_affliction_date"]]
    """Content-specific field names for ordering."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .ramp_type import RampType

__all__ = ["RampCreateParams", "Body"]


class RampCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    source_ramp_id: Required[str]
    """Identifier of the source network resource to associate as a ramp."""

    type: Required[RampType]
    """
    The type of network connection (ramp) linking a CF1 Site to Cloudflare's
    network.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .export_list_params import ExportListParams as ExportListParams
from .content_list_params import ContentListParams as ContentListParams
from .export_get_response import ExportGetResponse as ExportGetResponse
from .finding_list_params import FindingListParams as FindingListParams
from .export_list_response import ExportListResponse as ExportListResponse
from .finding_get_response import FindingGetResponse as FindingGetResponse
from .webhook_get_response import WebhookGetResponse as WebhookGetResponse
from .content_export_params import ContentExportParams as ContentExportParams
from .content_list_response import ContentListResponse as ContentListResponse
from .finding_export_params import FindingExportParams as FindingExportParams
from .finding_ignore_params import FindingIgnoreParams as FindingIgnoreParams
from .finding_list_response import FindingListResponse as FindingListResponse
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_list_response import WebhookListResponse as WebhookListResponse
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .content_export_response import ContentExportResponse as ContentExportResponse
from .finding_export_response import FindingExportResponse as FindingExportResponse
from .finding_ignore_response import FindingIgnoreResponse as FindingIgnoreResponse
from .finding_unignore_params import FindingUnignoreParams as FindingUnignoreParams
from .webhook_create_response import WebhookCreateResponse as WebhookCreateResponse
from .webhook_delete_response import WebhookDeleteResponse as WebhookDeleteResponse
from .webhook_evaluate_params import WebhookEvaluateParams as WebhookEvaluateParams
from .webhook_update_response import WebhookUpdateResponse as WebhookUpdateResponse
from .finding_type_list_params import FindingTypeListParams as FindingTypeListParams
from .finding_type_get_response import FindingTypeGetResponse as FindingTypeGetResponse
from .finding_unignore_response import FindingUnignoreResponse as FindingUnignoreResponse
from .webhook_evaluate_response import WebhookEvaluateResponse as WebhookEvaluateResponse
from .finding_type_list_response import FindingTypeListResponse as FindingTypeListResponse
from .finding_tune_severity_params import FindingTuneSeverityParams as FindingTuneSeverityParams
from .finding_tune_severity_response import FindingTuneSeverityResponse as FindingTuneSeverityResponse
from .finding_reset_severity_response import FindingResetSeverityResponse as FindingResetSeverityResponse
from .webhook_evaluate_existing_response import WebhookEvaluateExistingResponse as WebhookEvaluateExistingResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RemediationTypeListParams"]


class RemediationTypeListParams(TypedDict, total=False):
    account_id: Required[str]

    cursor: str
    """A cursor for pagination."""

    integration_id: str
    """Filter by an integration ID"""

    page: int
    """A page number within the paginated result set."""

    per_page: int
    """Number of results to return per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ......_models import BaseModel

__all__ = [
    "InstanceGetResponse",
    "Asset",
    "AssetCategory",
    "AssetField",
    "DLPContext",
    "Remediation",
    "Webhook",
    "WebhookLatestJob",
]


class AssetCategory(BaseModel):
    """Category information for an asset."""

    service: Optional[str] = None
    """The specific service within the vendor the asset is part of (often none).

    Example - AWS is the vendor, S3 is the service.
    """

    type: str
    """The type of asset."""

    vendor: str
    """The vendor the asset is part of."""

    id: Optional[str] = None
    """Unique identifier for the asset category."""


class AssetField(BaseModel):
    """Additional field information for an asset."""

    name: str
    """The name of the field."""

    value: str
    """The value of the field."""

    link: Optional[str] = None
    """Optional link associated with the field."""


class Asset(BaseModel):
    """Asset information including metadata and categorization."""

    category: AssetCategory
    """Category information for an asset."""

    external_id: str
    """External identifier from the source system."""

    fields: List[AssetField]
    """The fields associated with the asset."""

    name: str
    """Human-readable name of the asset."""

    id: Optional[str] = None
    """Unique identifier for the asset."""

    link: Optional[str] = None
    """Direct link to the asset."""


class DLPContext(BaseModel):
    """DLP context information for a finding."""

    created: datetime
    """When the DLP context was created."""

    entry_ids: List[str]
    """DLP Entry IDs."""

    profile_id: str
    """DLP Profile ID."""

    updated: datetime
    """When the DLP context was last updated."""

    id: Optional[str] = None
    """Unique identifier for the DLP context."""

    deleted: Optional[datetime] = None
    """When the DLP context was deleted."""

    match_context_max_extent: Optional[int] = None
    """DLP Right Boundary of match context."""

    match_context_min_extent: Optional[int] = None
    """DLP Left Boundary of match context."""

    match_context_payload: Optional[Dict[str, object]] = None
    """DLP Match context payload that matched the profile in question."""


class Remediation(BaseModel):
    """Summary information about a remediation job."""

    id: str
    """Unique identifier for the remediation job."""

    created_at: datetime
    """When the remediation job was created."""

    stale: bool
    """
    Whether this remediation job is stale (created before the finding instance's
    affliction_date).
    """

    status: Literal["pending", "processing", "completed", "failed", "validating"]
    """Status of a remediation job."""


class WebhookLatestJob(BaseModel):
    """The most recent webhook job for this webhook configuration."""

    id: str
    """Unique identifier for the webhook job."""

    created_at: datetime
    """When the webhook job was created."""

    stale: bool
    """
    Whether this webhook job is stale (created before the finding instance's current
    affliction_date).
    """

    status: Literal["pending", "processing", "completed"]
    """Current status of the webhook job."""


class Webhook(BaseModel):
    """
    Summary of the most recent webhook job invocation for a specific webhook configuration.
    """

    latest_job: WebhookLatestJob
    """The most recent webhook job for this webhook configuration."""

    webhook_id: str
    """Unique identifier for the webhook configuration."""

    webhook_label: str
    """Account-specified display label for the webhook configuration."""


class InstanceGetResponse(BaseModel):
    """A specific instance of a security finding.

    In the API interface, we refer to the 'finding' table in our DB as finding instances, optimized for the p99 use case.
    """

    affliction_date: datetime
    """When this specific instance was identified."""

    asset: Asset
    """Asset information including metadata and categorization."""

    dlp_contexts: List[DLPContext]
    """DLP context information if this is a content finding."""

    remediations: List[Remediation]
    """
    A list of the 10 most recent remediation jobs for this finding instance, ordered
    by creation time (most recent first). The 'stale' field indicates whether the
    remediation job was created before the finding instance's affliction_date (true)
    or after it (false). If there has never been a remediation job for this finding
    instance, this field will be an empty array.
    """

    webhooks: List[Webhook]
    """
    The most recent webhook job invocation for each webhook configuration associated
    with this finding instance. Each entry represents the latest job (any status)
    per webhook config. The 'stale' field indicates whether the job was invoked
    before the finding instance's current affliction_date. If no webhook jobs have
    been created, this field will be an empty array.
    """

    id: Optional[str] = None
    """Unique identifier for the finding instance."""

    is_archived: Optional[bool] = None
    """Whether this finding instance has been archived."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ......_utils import PropertyInfo

__all__ = ["JobListParams"]


class JobListParams(TypedDict, total=False):
    account_id: Required[str]

    cursor: str
    """A cursor for pagination."""

    direction: Literal["asc", "desc"]
    """Direction to order results."""

    integration_id: str
    """Filter by an integration ID"""

    max_updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view remediations updated on or before the max updated datetime.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    min_updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view remediations updated on or after the min updated datetime.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    order: Literal[
        "created_at",
        "affliction_date",
        "integration_name",
        "status",
        "last_updated_at",
        "asset_name",
        "finding_type_name",
    ]
    """An optional param to sort the results by the given field."""

    page: int
    """A page number within the paginated result set."""

    per_page: int
    """Number of results to return per page."""

    search: str
    """A search term."""

    status: Literal["pending", "processing", "completed", "failed", "validating"]
    """Filter to view remediations with the given status."""

    triggered_by_actor: List[Literal["user", "account_token"]]
    """Filter remediations by what kind of actor triggered them.

    Supports multiple comma-separated values.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tenant_custom_nameservers import (
    TenantCustomNameserversResource,
    AsyncTenantCustomNameserversResource,
    TenantCustomNameserversResourceWithRawResponse,
    AsyncTenantCustomNameserversResourceWithRawResponse,
    TenantCustomNameserversResourceWithStreamingResponse,
    AsyncTenantCustomNameserversResourceWithStreamingResponse,
)

__all__ = [
    "TenantCustomNameserversResource",
    "AsyncTenantCustomNameserversResource",
    "TenantCustomNameserversResourceWithRawResponse",
    "AsyncTenantCustomNameserversResourceWithRawResponse",
    "TenantCustomNameserversResourceWithStreamingResponse",
    "AsyncTenantCustomNameserversResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["FindingListParams"]


class FindingListParams(TypedDict, total=False):
    account_id: Required[str]

    cursor: str
    """A cursor for pagination.

    Obtained from the `result_info.cursor` field of a previous response.
    """

    direction: Literal["asc", "desc"]
    """Direction to order results."""

    finding_type_ids: str
    """A comma separated list of UUIDs identifying the finding type(s)."""

    ignored: bool
    """Filter for only the ignored findings. Set to false to only see "active" items"""

    integration_id: str
    """Filter by an integration ID"""

    max_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view findings that occurred on or before the affliction date.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    min_affliction_date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter to view findings that occurred on or after the affliction date.

    Can be a date-time in ISO 8601 format or an epoch timestamp.
    """

    observation: Literal["Activity", "Insight", "Issue"]
    """Filter by observation type of the finding"""

    order: Literal["finding.name", "instance_count", "integration.name", "latest_affliction_date", "severity"]
    """Which field to use when ordering the findings."""

    page: int
    """A page number within the paginated result set."""

    per_page: int
    """Number of results to return per page."""

    product: Literal["Cloud", "Saas"]
    """Filter by product category of the finding"""

    search: str
    """A search term."""

    severity: Literal["Critical", "High", "Medium", "Low"]
    """Filter by severity"""

    type: Literal["Content", "Posture"]
    """Filter by type of the finding"""

    vendor: Literal[
        "ANTHROPIC",
        "AWS",
        "BITBUCKET",
        "BOX",
        "CONFLUENCE",
        "DROPBOX",
        "GITHUB",
        "GOOGLE_CLOUD_PLATFORM",
        "GOOGLE_WORKSPACE",
        "JIRA",
        "MICROSOFT",
        "MICROSOFT_INTERNAL",
        "OPENAI",
        "SALESFORCE",
        "SERVICENOW",
        "SLACK",
    ]
    """Filter by vendor"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional
from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["IntegrationCreateParams"]


class IntegrationCreateParams(TypedDict, total=False):
    account_id: Required[str]

    application: Required[
        Literal[
            "ANTHROPIC",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SLACK",
        ]
    ]
    """Vendor/application slug (e.g., GOOGLE_WORKSPACE).

    - `ANTHROPIC` - ANTHROPIC
    - `BITBUCKET` - BITBUCKET
    - `BOX` - BOX
    - `CONFLUENCE` - CONFLUENCE
    - `DROPBOX` - DROPBOX
    - `GITHUB` - GITHUB
    - `GOOGLE_CLOUD_PLATFORM` - GOOGLE_CLOUD_PLATFORM
    - `GOOGLE_WORKSPACE` - GOOGLE_WORKSPACE
    - `JIRA` - JIRA
    - `MICROSOFT_INTERNAL` - MICROSOFT_INTERNAL
    - `OPENAI` - OPENAI
    - `SALESFORCE` - SALESFORCE
    - `SLACK` - SLACK
    """

    credentials: Required[Dict[str, object]]
    """Credentials for the integration."""

    name: Required[str]
    """Name of the integration."""

    auth_method: Optional[str]
    """Authentication method slug (uses default if omitted)."""

    dlp_profiles: SequenceNotStr[str]
    """List of DLP profile IDs to associate."""

    permissions: SequenceNotStr[str]
    """List of permission scopes (uses policy defaults if empty)."""

    use_cases: List[Literal["casb", "ces", "auto_remediation"]]
    """
    List of use case or feature slugs to enroll (e.g., ['casb', 'ces',
    'auto_remediation']).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.resource_sharing import recipient_get_params, recipient_list_params, recipient_create_params
from ...types.resource_sharing.recipient_get_response import RecipientGetResponse
from ...types.resource_sharing.recipient_list_response import RecipientListResponse
from ...types.resource_sharing.recipient_create_response import RecipientCreateResponse
from ...types.resource_sharing.recipient_delete_response import RecipientDeleteResponse

__all__ = ["RecipientsResource", "AsyncRecipientsResource"]


class RecipientsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RecipientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RecipientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecipientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RecipientsResourceWithStreamingResponse(self)

    def create(
        self,
        share_id: str,
        *,
        path_account_id: str,
        body_account_id: str | Omit = omit,
        organization_id: str | Omit = omit,
        recipient_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientCreateResponse]:
        """
        Adds a recipient to a resource share, granting them access to the shared
        resources.

        Args:
          path_account_id: Account identifier.

          share_id: Share identifier tag.

          body_account_id: Deprecated alias for `recipient_account_id`. Use `recipient_account_id` instead.
              The body field collided with the URL path parameter of the same name, which
              prevented SDK generators from distinguishing the source account (in the URL)
              from the recipient account (in the body). Both names will continue to be
              accepted until 2027-05-26 (see `x-sunset`).

          organization_id: Organization identifier.

          recipient_account_id: The account that will receive the share.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_account_id:
            raise ValueError(f"Expected a non-empty value for `path_account_id` but received {path_account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._post(
            path_template(
                "/accounts/{path_account_id}/shares/{share_id}/recipients",
                path_account_id=path_account_id,
                share_id=share_id,
            ),
            body=maybe_transform(
                {
                    "body_account_id": body_account_id,
                    "organization_id": organization_id,
                    "recipient_account_id": recipient_account_id,
                },
                recipient_create_params.RecipientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientCreateResponse]], ResultWrapper[RecipientCreateResponse]),
        )

    def list(
        self,
        share_id: str,
        *,
        account_id: str,
        include_resources: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[RecipientListResponse]:
        """
        List share recipients by share ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          include_resources: Include resources in the response.

          page: Page number. Defaults to `1` when `per_page` is supplied without `page`. May be
              omitted entirely along with `per_page` to receive a non-paginated response.

          per_page: Number of objects to return per page. Defaults to `20` when `page` is supplied
              without `per_page`. May be omitted entirely along with `page` to receive a
              non-paginated response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients", account_id=account_id, share_id=share_id
            ),
            page=SyncV4PagePaginationArray[RecipientListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_resources": include_resources,
                        "page": page,
                        "per_page": per_page,
                    },
                    recipient_list_params.RecipientListParams,
                ),
            ),
            model=RecipientListResponse,
        )

    def delete(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientDeleteResponse]:
        """
        Deletion is not immediate, an updated share recipient object with a new status
        will be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientDeleteResponse]], ResultWrapper[RecipientDeleteResponse]),
        )

    def get(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        include_resources: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientGetResponse]:
        """
        Get share recipient by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          include_resources: Include resources in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include_resources": include_resources}, recipient_get_params.RecipientGetParams
                ),
                post_parser=ResultWrapper[Optional[RecipientGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientGetResponse]], ResultWrapper[RecipientGetResponse]),
        )


class AsyncRecipientsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRecipientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRecipientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecipientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRecipientsResourceWithStreamingResponse(self)

    async def create(
        self,
        share_id: str,
        *,
        path_account_id: str,
        body_account_id: str | Omit = omit,
        organization_id: str | Omit = omit,
        recipient_account_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientCreateResponse]:
        """
        Adds a recipient to a resource share, granting them access to the shared
        resources.

        Args:
          path_account_id: Account identifier.

          share_id: Share identifier tag.

          body_account_id: Deprecated alias for `recipient_account_id`. Use `recipient_account_id` instead.
              The body field collided with the URL path parameter of the same name, which
              prevented SDK generators from distinguishing the source account (in the URL)
              from the recipient account (in the body). Both names will continue to be
              accepted until 2027-05-26 (see `x-sunset`).

          organization_id: Organization identifier.

          recipient_account_id: The account that will receive the share.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_account_id:
            raise ValueError(f"Expected a non-empty value for `path_account_id` but received {path_account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return await self._post(
            path_template(
                "/accounts/{path_account_id}/shares/{share_id}/recipients",
                path_account_id=path_account_id,
                share_id=share_id,
            ),
            body=await async_maybe_transform(
                {
                    "body_account_id": body_account_id,
                    "organization_id": organization_id,
                    "recipient_account_id": recipient_account_id,
                },
                recipient_create_params.RecipientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientCreateResponse]], ResultWrapper[RecipientCreateResponse]),
        )

    def list(
        self,
        share_id: str,
        *,
        account_id: str,
        include_resources: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RecipientListResponse, AsyncV4PagePaginationArray[RecipientListResponse]]:
        """
        List share recipients by share ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          include_resources: Include resources in the response.

          page: Page number. Defaults to `1` when `per_page` is supplied without `page`. May be
              omitted entirely along with `per_page` to receive a non-paginated response.

          per_page: Number of objects to return per page. Defaults to `20` when `page` is supplied
              without `per_page`. May be omitted entirely along with `page` to receive a
              non-paginated response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients", account_id=account_id, share_id=share_id
            ),
            page=AsyncV4PagePaginationArray[RecipientListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_resources": include_resources,
                        "page": page,
                        "per_page": per_page,
                    },
                    recipient_list_params.RecipientListParams,
                ),
            ),
            model=RecipientListResponse,
        )

    async def delete(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientDeleteResponse]:
        """
        Deletion is not immediate, an updated share recipient object with a new status
        will be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RecipientDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientDeleteResponse]], ResultWrapper[RecipientDeleteResponse]),
        )

    async def get(
        self,
        recipient_id: str,
        *,
        account_id: str,
        share_id: str,
        include_resources: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RecipientGetResponse]:
        """
        Get share recipient by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          recipient_id: Share Recipient identifier tag.

          include_resources: Include resources in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not recipient_id:
            raise ValueError(f"Expected a non-empty value for `recipient_id` but received {recipient_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/recipients/{recipient_id}",
                account_id=account_id,
                share_id=share_id,
                recipient_id=recipient_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_resources": include_resources}, recipient_get_params.RecipientGetParams
                ),
                post_parser=ResultWrapper[Optional[RecipientGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RecipientGetResponse]], ResultWrapper[RecipientGetResponse]),
        )


class RecipientsResourceWithRawResponse:
    def __init__(self, recipients: RecipientsResource) -> None:
        self._recipients = recipients

        self.create = to_raw_response_wrapper(
            recipients.create,
        )
        self.list = to_raw_response_wrapper(
            recipients.list,
        )
        self.delete = to_raw_response_wrapper(
            recipients.delete,
        )
        self.get = to_raw_response_wrapper(
            recipients.get,
        )


class AsyncRecipientsResourceWithRawResponse:
    def __init__(self, recipients: AsyncRecipientsResource) -> None:
        self._recipients = recipients

        self.create = async_to_raw_response_wrapper(
            recipients.create,
        )
        self.list = async_to_raw_response_wrapper(
            recipients.list,
        )
        self.delete = async_to_raw_response_wrapper(
            recipients.delete,
        )
        self.get = async_to_raw_response_wrapper(
            recipients.get,
        )


class RecipientsResourceWithStreamingResponse:
    def __init__(self, recipients: RecipientsResource) -> None:
        self._recipients = recipients

        self.create = to_streamed_response_wrapper(
            recipients.create,
        )
        self.list = to_streamed_response_wrapper(
            recipients.list,
        )
        self.delete = to_streamed_response_wrapper(
            recipients.delete,
        )
        self.get = to_streamed_response_wrapper(
            recipients.get,
        )


class AsyncRecipientsResourceWithStreamingResponse:
    def __init__(self, recipients: AsyncRecipientsResource) -> None:
        self._recipients = recipients

        self.create = async_to_streamed_response_wrapper(
            recipients.create,
        )
        self.list = async_to_streamed_response_wrapper(
            recipients.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            recipients.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            recipients.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["RobotBulkGetParams"]


class RobotBulkGetParams(TypedDict, total=False):
    zone_id: Required[str]

    body: Required[SequenceNotStr[str]]
    """Array of domain hostnames to fetch robots.txt for.

    Each domain must end with the zone name. Maximum 25 domains per request.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["SensitivityGroupListResponse", "Level"]


class Level(BaseModel):
    id: str

    created_at: datetime

    name: str

    updated_at: datetime

    description: Optional[str] = None


class SensitivityGroupListResponse(BaseModel):
    id: str

    created_at: datetime

    levels: List[Level]

    name: str

    updated_at: datetime

    description: Optional[str] = None

    template_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os

import httpx

from ...._files import read_file_content
from ...._types import (
    Body,
    Omit,
    Query,
    Headers,
    NotGiven,
    BinaryTypes,
    FileContent,
    omit,
    not_given,
)
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.logs.log_explorer.query_sql_response import QuerySqlResponse

__all__ = ["QueryResource", "AsyncQueryResource"]


class QueryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QueryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return QueryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return QueryResourceWithStreamingResponse(self)

    def sql(
        self,
        body: FileContent | BinaryTypes,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[QuerySqlResponse]:
        """
        Run a SQL query against account or zone-level datasets.

        Timestamp fields are RFC3339 strings. Filter with: WHERE {timestamp_field} >=
        now() - INTERVAL '30' DAY WHERE {timestamp_field} >= '2026-04-01T00:00:00Z'
        WHERE {timestamp_field} BETWEEN '2026-04-01T00:00:00Z' AND
        '2026-04-30T23:59:59Z'

        List configured account or zone datasets to see enabled account or zone-level
        datasets. Zone-level datasets will not appear here. List available account or
        zone datasets to inspect their schemas and timestamp fields.

        For more information about the datasets, and the meaning of each field, check
        out https://developers.cloudflare.com/logs/logpush/logpush-job/datasets/

        Args:
          body: SQL query to execute.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        extra_headers = {"Content-Type": "text/plain", **(extra_headers or {})}
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logs/explorer/query/sql",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncSinglePage[QuerySqlResponse],
            content=read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=QuerySqlResponse,
            method="post",
        )


class AsyncQueryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQueryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQueryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncQueryResourceWithStreamingResponse(self)

    def sql(
        self,
        body: FileContent | BinaryTypes,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[QuerySqlResponse, AsyncSinglePage[QuerySqlResponse]]:
        """
        Run a SQL query against account or zone-level datasets.

        Timestamp fields are RFC3339 strings. Filter with: WHERE {timestamp_field} >=
        now() - INTERVAL '30' DAY WHERE {timestamp_field} >= '2026-04-01T00:00:00Z'
        WHERE {timestamp_field} BETWEEN '2026-04-01T00:00:00Z' AND
        '2026-04-30T23:59:59Z'

        List configured account or zone datasets to see enabled account or zone-level
        datasets. Zone-level datasets will not appear here. List available account or
        zone datasets to inspect their schemas and timestamp fields.

        For more information about the datasets, and the meaning of each field, check
        out https://developers.cloudflare.com/logs/logpush/logpush-job/datasets/

        Args:
          body: SQL query to execute.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        extra_headers = {"Content-Type": "text/plain", **(extra_headers or {})}
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logs/explorer/query/sql",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncSinglePage[QuerySqlResponse],
            content=read_file_content(body) if isinstance(body, os.PathLike) else body,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=QuerySqlResponse,
            method="post",
        )


class QueryResourceWithRawResponse:
    def __init__(self, query: QueryResource) -> None:
        self._query = query

        self.sql = to_raw_response_wrapper(
            query.sql,
        )


class AsyncQueryResourceWithRawResponse:
    def __init__(self, query: AsyncQueryResource) -> None:
        self._query = query

        self.sql = async_to_raw_response_wrapper(
            query.sql,
        )


class QueryResourceWithStreamingResponse:
    def __init__(self, query: QueryResource) -> None:
        self._query = query

        self.sql = to_streamed_response_wrapper(
            query.sql,
        )


class AsyncQueryResourceWithStreamingResponse:
    def __init__(self, query: AsyncQueryResource) -> None:
        self._query = query

        self.sql = async_to_streamed_response_wrapper(
            query.sql,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ......_types import SequenceNotStr

__all__ = ["JobCreateParams"]


class JobCreateParams(TypedDict, total=False):
    account_id: Required[str]

    finding_instance_ids: Required[SequenceNotStr[str]]
    """UUIDs identifying Finding Instances."""

    remediation_type_id: Required[str]
    """A UUID identifying this Remediation Type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["RelayUpdateParams", "Config", "ConfigUpstreams", "ConfigUpstreamsUpstream"]


class RelayUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account identifier."""

    config: Config

    name: str


class ConfigUpstreamsUpstream(TypedDict, total=False):
    """A single upstream MOQT server publisher."""

    url: Required[str]
    """Upstream MOQT server publisher URL.

    Must be an absolute URL with a host and a scheme crique can dial: moqt:// (raw
    QUIC) or https:// (WebTransport). Validated on update (PUT); rejected
    with 21013.
    """


class ConfigUpstreams(TypedDict, total=False):
    """
    Upstreams are external MOQT server publishers that a relay falls back
    to when it has no local publisher for a requested namespace/track.
    """

    enabled: bool

    upstreams: Iterable[ConfigUpstreamsUpstream]
    """Ordered list of upstream MOQT server publishers.

    Each entry is an object (not a bare string) so per-upstream configuration can be
    added in the future without another breaking change.
    """


class Config(TypedDict, total=False):
    upstreams: ConfigUpstreams
    """
    Upstreams are external MOQT server publishers that a relay falls back to when it
    has no local publisher for a requested namespace/track.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["TopAsesParams"]


class TopAsesParams(TypedDict, total=False):
    asn: SequenceNotStr[str]
    """Filters results by Autonomous System.

    Specify one or more Autonomous System Numbers (ASNs) as a comma-separated list.
    Prefix with `-` to exclude ASNs from results. For example, `-174, 3356` excludes
    results from AS174, but includes results from AS3356.
    """

    continent: SequenceNotStr[str]
    """Filters results by continent.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    continents from results. For example, `-EU,NA` excludes results from EU, but
    includes results from NA.
    """

    date_end: Annotated[SequenceNotStr[Union[str, datetime]], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive).

    Alternative to `dateRange`; provide together with `dateStart`. When requesting
    comparison series, every series must resolve to the same duration as the main
    series. Each `dateStart`/`dateEnd` is floored to the nearest 15 minutes before
    evaluation, so windows whose durations match only before alignment may be
    rejected.
    """

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    limit: int
    """Limits the number of objects returned in the response."""

    location: SequenceNotStr[str]
    """Filters results by location.

    Specify a comma-separated list of alpha-2 codes. Prefix with `-` to exclude
    locations from results. For example, `-US,PT` excludes results from the US, but
    includes results from PT.
    """

    name: SequenceNotStr[str]
    """Array of names used to label the series in the response."""

    order_by: Annotated[
        Literal[
            "BANDWIDTH_DOWNLOAD", "BANDWIDTH_UPLOAD", "LATENCY_IDLE", "LATENCY_LOADED", "JITTER_IDLE", "JITTER_LOADED"
        ],
        PropertyInfo(alias="orderBy"),
    ]
    """Specifies the metric to order the results by."""

    reverse: bool
    """Reverses the order of results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource

__all__ = ["RemediationsResource", "AsyncRemediationsResource"]


class RemediationsResource(SyncAPIResource):
    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RemediationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RemediationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RemediationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RemediationsResourceWithStreamingResponse(self)


class AsyncRemediationsResource(AsyncAPIResource):
    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRemediationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRemediationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRemediationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRemediationsResourceWithStreamingResponse(self)


class RemediationsResourceWithRawResponse:
    def __init__(self, remediations: RemediationsResource) -> None:
        self._remediations = remediations

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._remediations.jobs)


class AsyncRemediationsResourceWithRawResponse:
    def __init__(self, remediations: AsyncRemediationsResource) -> None:
        self._remediations = remediations

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._remediations.jobs)


class RemediationsResourceWithStreamingResponse:
    def __init__(self, remediations: RemediationsResource) -> None:
        self._remediations = remediations

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._remediations.jobs)


class AsyncRemediationsResourceWithStreamingResponse:
    def __init__(self, remediations: AsyncRemediationsResource) -> None:
        self._remediations = remediations

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._remediations.jobs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AccountEditParams", "ZoneDefaults", "ZoneDefaultsInternalDNS", "ZoneDefaultsNameservers", "ZoneDefaultsSOA"]


class AccountEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    enforce_dns_only: bool
    """
    When enabled, forces all proxied DNS records in the account to behave as
    DNS-only at the edge, regardless of each record's individual proxy setting. Note
    that this account-level override does not modify the records themselves; it only
    affects how they are served at the edge. See more on
    [Enforce DNS-only](https://developers.cloudflare.com/dns/proxy-status/enforce-dns-only).
    """

    zone_defaults: ZoneDefaults


class ZoneDefaultsInternalDNS(TypedDict, total=False):
    """Settings for this internal zone."""

    reference_zone_id: str
    """The ID of the zone to fallback to."""


class ZoneDefaultsNameservers(TypedDict, total=False):
    """
    Settings determining the nameservers through which the zone should be available.
    """

    type: Literal[
        "cloudflare.standard", "cloudflare.advanced", "cloudflare.standard.random", "custom.account", "custom.tenant"
    ]
    """Nameserver type"""


class ZoneDefaultsSOA(TypedDict, total=False):
    """Components of the zone's SOA record."""

    expire: float
    """
    Time in seconds of being unable to query the primary server after which
    secondary servers should stop serving the zone.
    """

    min_ttl: float
    """The time to live (TTL) for negative caching of records within the zone."""

    mname: Optional[str]
    """The primary nameserver, which may be used for outbound zone transfers.

    If null, a Cloudflare-assigned value will be used.
    """

    refresh: float
    """
    Time in seconds after which secondary servers should re-check the SOA record to
    see if the zone has been updated.
    """

    retry: float
    """
    Time in seconds after which secondary servers should retry queries after the
    primary server was unresponsive.
    """

    rname: str
    """
    The email address of the zone administrator, with the first label representing
    the local part of the email address.
    """

    ttl: float
    """The time to live (TTL) of the SOA record itself."""


class ZoneDefaults(TypedDict, total=False):
    flatten_all_cnames: bool
    """Whether to flatten all CNAME records in the zone.

    Note that, due to DNS limitations, a CNAME record at the zone apex will always
    be flattened.
    """

    foundation_dns: bool
    """Deprecated. Use nameservers.type to configure Advanced Nameservers."""

    internal_dns: ZoneDefaultsInternalDNS
    """Settings for this internal zone."""

    multi_provider: bool
    """
    Whether to enable multi-provider DNS, which causes Cloudflare to activate the
    zone even when non-Cloudflare NS records exist, and to respect NS records at the
    zone apex during outbound zone transfers.
    """

    nameservers: ZoneDefaultsNameservers
    """
    Settings determining the nameservers through which the zone should be available.
    """

    ns_ttl: float
    """The time to live (TTL) of the zone's nameserver (NS) records."""

    secondary_overrides: bool
    """
    Allows a Secondary DNS zone to use (proxied) override records and CNAME
    flattening at the zone apex.
    """

    soa: ZoneDefaultsSOA
    """Components of the zone's SOA record."""

    zone_mode: Literal["standard", "cdn_only", "dns_only"]
    """Whether the zone mode is a regular or CDN/DNS only zone."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ......_types import SequenceNotStr

__all__ = ["InstanceUnarchiveParams"]


class InstanceUnarchiveParams(TypedDict, total=False):
    account_id: Required[str]

    check_instances: Required[SequenceNotStr[str]]
    """A list of finding instance IDs to pass along."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.zero_trust.casb.applications.auth_method_list_response import AuthMethodListResponse

__all__ = ["AuthMethodsResource", "AsyncAuthMethodsResource"]


class AuthMethodsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthMethodsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AuthMethodsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthMethodsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AuthMethodsResourceWithStreamingResponse(self)

    def list(
        self,
        application_id: Literal[
            "ANTHROPIC",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SLACK",
        ],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthMethodListResponse:
        """
        Returns available auth methods for the specified vendor, including credential
        schema, instructions, and example payloads. Use this to understand what
        credentials are required before calling POST /v2/integrations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/one/applications/{application_id}/auth-methods",
                account_id=account_id,
                application_id=application_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthMethodListResponse,
        )


class AsyncAuthMethodsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthMethodsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthMethodsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthMethodsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAuthMethodsResourceWithStreamingResponse(self)

    async def list(
        self,
        application_id: Literal[
            "ANTHROPIC",
            "BITBUCKET",
            "BOX",
            "CONFLUENCE",
            "DROPBOX",
            "GITHUB",
            "GOOGLE_CLOUD_PLATFORM",
            "GOOGLE_WORKSPACE",
            "JIRA",
            "MICROSOFT_INTERNAL",
            "OPENAI",
            "SALESFORCE",
            "SLACK",
        ],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthMethodListResponse:
        """
        Returns available auth methods for the specified vendor, including credential
        schema, instructions, and example payloads. Use this to understand what
        credentials are required before calling POST /v2/integrations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not application_id:
            raise ValueError(f"Expected a non-empty value for `application_id` but received {application_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/one/applications/{application_id}/auth-methods",
                account_id=account_id,
                application_id=application_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthMethodListResponse,
        )


class AuthMethodsResourceWithRawResponse:
    def __init__(self, auth_methods: AuthMethodsResource) -> None:
        self._auth_methods = auth_methods

        self.list = to_raw_response_wrapper(
            auth_methods.list,
        )


class AsyncAuthMethodsResourceWithRawResponse:
    def __init__(self, auth_methods: AsyncAuthMethodsResource) -> None:
        self._auth_methods = auth_methods

        self.list = async_to_raw_response_wrapper(
            auth_methods.list,
        )


class AuthMethodsResourceWithStreamingResponse:
    def __init__(self, auth_methods: AuthMethodsResource) -> None:
        self._auth_methods = auth_methods

        self.list = to_streamed_response_wrapper(
            auth_methods.list,
        )


class AsyncAuthMethodsResourceWithStreamingResponse:
    def __init__(self, auth_methods: AsyncAuthMethodsResource) -> None:
        self._auth_methods = auth_methods

        self.list = async_to_streamed_response_wrapper(
            auth_methods.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MitigationListResponse", "Mitigation"]


class Mitigation(BaseModel):
    id: str
    """ID of remediation."""

    effective_date: str
    """Date when the mitigation will become active.

    Time in RFC 3339 format (https://www.rfc-editor.org/rfc/rfc3339.html)
    """

    entity_id: str

    entity_type: Literal["url_pattern", "account", "zone"]
    """The type of entity targeted by a mitigation."""

    status: Literal["pending", "active", "in_review", "cancelled", "removed"]
    """The status of a mitigation"""

    type: Literal[
        "account_suspend",
        "copyright_interstitial",
        "geo_block",
        "legal_block",
        "malware_interstitial",
        "misleading_interstitial",
        "network_block",
        "phishing_interstitial",
        "playfairite_enforce",
        "r2_takedown_account",
        "r2_takedown_bucket",
        "r2_takedown_object",
        "rate_limit_cache",
        "redirect_video_stream",
        "zone_fint",
        "registrar_freeze",
        "registrar_parking",
        "stream_block_account",
        "user_suspend",
        "workers_takedown_by_zone_id",
    ]
    """The type of mitigation applied to a reported entity."""


class MitigationListResponse(BaseModel):
    mitigations: List[Mitigation]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["IndicatorListParams", "Search"]


class IndicatorListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    dataset_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="datasetIds")]
    """
    Dataset UUIDs to scope to (repeat the param for multiple), or 'all' / '\\**' for
    every readable indicator dataset. Omit to search all readable datasets.
    """

    indicator_type: Annotated[str, PropertyInfo(alias="indicatorType")]

    page: float

    page_size: Annotated[float, PropertyInfo(alias="pageSize")]

    related_event: Annotated[SequenceNotStr[str], PropertyInfo(alias="relatedEvent")]
    """Filter indicators by related event UUID(s).

    Multiple UUIDs can be provided by repeating the parameter.
    """

    search: Iterable[Search]
    """Structured search as a JSON array of {field, op, value} objects.

    Searchable fields: value, indicatorType. Multiple conditions are AND'd together.
    Max 10 conditions per request.
    """


class Search(TypedDict, total=False):
    field: Required[Literal["value", "indicatorType", "uuid"]]
    """The indicator field to search on. Allowed: value, indicatorType, uuid."""

    op: Required[
        Literal["equals", "not", "gt", "gte", "lt", "lte", "like", "contains", "startsWith", "endsWith", "in", "find"]
    ]
    """Search operator.

    Use 'in' for bulk lookup of up to 100 values at once, e.g. {field:'value',
    op:'in', value:['evil.com','bad.org']}.
    """

    value: Required[Union[str, SequenceNotStr[str]]]
    """Search value.

    String for most operators. Array of strings for 'in' operator (max 100 items).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["DatasetListParams"]


class DatasetListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    include_deleted: Annotated[bool, PropertyInfo(alias="includeDeleted")]
    """When true, include soft-deleted datasets in the response.

    Each item includes a `deletedAt` field (ISO 8601 or null). Default: false.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from ......_types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ......_utils import path_template, maybe_transform, async_maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_wrappers import ResultWrapper
from ......pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.access.ai_controls.mcp import portal_list_params, portal_create_params, portal_update_params
from ......types.zero_trust.access.ai_controls.mcp.portal_list_response import PortalListResponse
from ......types.zero_trust.access.ai_controls.mcp.portal_read_response import PortalReadResponse
from ......types.zero_trust.access.ai_controls.mcp.portal_create_response import PortalCreateResponse
from ......types.zero_trust.access.ai_controls.mcp.portal_delete_response import PortalDeleteResponse
from ......types.zero_trust.access.ai_controls.mcp.portal_update_response import PortalUpdateResponse

__all__ = ["PortalsResource", "AsyncPortalsResource"]


class PortalsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PortalsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PortalsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PortalsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PortalsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        id: str,
        hostname: str,
        name: str,
        allow_code_mode: bool | Omit = omit,
        description: str | Omit = omit,
        secure_web_gateway: bool | Omit = omit,
        servers: Iterable[portal_create_params.Server] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalCreateResponse:
        """
        Creates a new MCP portal for managing AI tool access through Cloudflare Access.

        Args:
          id: portal id

          allow_code_mode: Allow remote code execution in Dynamic Workers (beta)

          secure_web_gateway: Route outbound MCP traffic through Zero Trust Secure Web Gateway

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals", account_id=account_id),
            body=maybe_transform(
                {
                    "id": id,
                    "hostname": hostname,
                    "name": name,
                    "allow_code_mode": allow_code_mode,
                    "description": description,
                    "secure_web_gateway": secure_web_gateway,
                    "servers": servers,
                },
                portal_create_params.PortalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalCreateResponse], ResultWrapper[PortalCreateResponse]),
        )

    def update(
        self,
        id: str,
        *,
        account_id: str,
        allow_code_mode: bool | Omit = omit,
        description: str | Omit = omit,
        hostname: str | Omit = omit,
        name: str | Omit = omit,
        secure_web_gateway: bool | Omit = omit,
        servers: Iterable[portal_update_params.Server] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalUpdateResponse:
        """
        Updates an MCP portal configuration.

        Args:
          id: portal id

          allow_code_mode: Allow remote code execution in Dynamic Workers (beta)

          secure_web_gateway: Route outbound MCP traffic through Zero Trust Secure Web Gateway

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals/{id}", account_id=account_id, id=id),
            body=maybe_transform(
                {
                    "allow_code_mode": allow_code_mode,
                    "description": description,
                    "hostname": hostname,
                    "name": name,
                    "secure_web_gateway": secure_web_gateway,
                    "servers": servers,
                },
                portal_update_params.PortalUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalUpdateResponse], ResultWrapper[PortalUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[PortalListResponse]:
        """
        Lists all MCP portals configured for the account.

        Args:
          search: Search by id, name, hostname

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals", account_id=account_id),
            page=SyncV4PagePaginationArray[PortalListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    portal_list_params.PortalListParams,
                ),
            ),
            model=PortalListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalDeleteResponse:
        """
        Deletes an MCP portal from the account.

        Args:
          id: portal id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalDeleteResponse], ResultWrapper[PortalDeleteResponse]),
        )

    def read(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalReadResponse:
        """
        Read the details of a single MCP Portal, including its configured servers.

        Args:
          id: portal id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalReadResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalReadResponse], ResultWrapper[PortalReadResponse]),
        )


class AsyncPortalsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPortalsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPortalsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPortalsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPortalsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        id: str,
        hostname: str,
        name: str,
        allow_code_mode: bool | Omit = omit,
        description: str | Omit = omit,
        secure_web_gateway: bool | Omit = omit,
        servers: Iterable[portal_create_params.Server] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalCreateResponse:
        """
        Creates a new MCP portal for managing AI tool access through Cloudflare Access.

        Args:
          id: portal id

          allow_code_mode: Allow remote code execution in Dynamic Workers (beta)

          secure_web_gateway: Route outbound MCP traffic through Zero Trust Secure Web Gateway

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "id": id,
                    "hostname": hostname,
                    "name": name,
                    "allow_code_mode": allow_code_mode,
                    "description": description,
                    "secure_web_gateway": secure_web_gateway,
                    "servers": servers,
                },
                portal_create_params.PortalCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalCreateResponse], ResultWrapper[PortalCreateResponse]),
        )

    async def update(
        self,
        id: str,
        *,
        account_id: str,
        allow_code_mode: bool | Omit = omit,
        description: str | Omit = omit,
        hostname: str | Omit = omit,
        name: str | Omit = omit,
        secure_web_gateway: bool | Omit = omit,
        servers: Iterable[portal_update_params.Server] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalUpdateResponse:
        """
        Updates an MCP portal configuration.

        Args:
          id: portal id

          allow_code_mode: Allow remote code execution in Dynamic Workers (beta)

          secure_web_gateway: Route outbound MCP traffic through Zero Trust Secure Web Gateway

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals/{id}", account_id=account_id, id=id),
            body=await async_maybe_transform(
                {
                    "allow_code_mode": allow_code_mode,
                    "description": description,
                    "hostname": hostname,
                    "name": name,
                    "secure_web_gateway": secure_web_gateway,
                    "servers": servers,
                },
                portal_update_params.PortalUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalUpdateResponse], ResultWrapper[PortalUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortalListResponse, AsyncV4PagePaginationArray[PortalListResponse]]:
        """
        Lists all MCP portals configured for the account.

        Args:
          search: Search by id, name, hostname

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals", account_id=account_id),
            page=AsyncV4PagePaginationArray[PortalListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    portal_list_params.PortalListParams,
                ),
            ),
            model=PortalListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalDeleteResponse:
        """
        Deletes an MCP portal from the account.

        Args:
          id: portal id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalDeleteResponse], ResultWrapper[PortalDeleteResponse]),
        )

    async def read(
        self,
        id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortalReadResponse:
        """
        Read the details of a single MCP Portal, including its configured servers.

        Args:
          id: portal id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/access/ai-controls/mcp/portals/{id}", account_id=account_id, id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PortalReadResponse]._unwrapper,
            ),
            cast_to=cast(Type[PortalReadResponse], ResultWrapper[PortalReadResponse]),
        )


class PortalsResourceWithRawResponse:
    def __init__(self, portals: PortalsResource) -> None:
        self._portals = portals

        self.create = to_raw_response_wrapper(
            portals.create,
        )
        self.update = to_raw_response_wrapper(
            portals.update,
        )
        self.list = to_raw_response_wrapper(
            portals.list,
        )
        self.delete = to_raw_response_wrapper(
            portals.delete,
        )
        self.read = to_raw_response_wrapper(
            portals.read,
        )


class AsyncPortalsResourceWithRawResponse:
    def __init__(self, portals: AsyncPortalsResource) -> None:
        self._portals = portals

        self.create = async_to_raw_response_wrapper(
            portals.create,
        )
        self.update = async_to_raw_response_wrapper(
            portals.update,
        )
        self.list = async_to_raw_response_wrapper(
            portals.list,
        )
        self.delete = async_to_raw_response_wrapper(
            portals.delete,
        )
        self.read = async_to_raw_response_wrapper(
            portals.read,
        )


class PortalsResourceWithStreamingResponse:
    def __init__(self, portals: PortalsResource) -> None:
        self._portals = portals

        self.create = to_streamed_response_wrapper(
            portals.create,
        )
        self.update = to_streamed_response_wrapper(
            portals.update,
        )
        self.list = to_streamed_response_wrapper(
            portals.list,
        )
        self.delete = to_streamed_response_wrapper(
            portals.delete,
        )
        self.read = to_streamed_response_wrapper(
            portals.read,
        )


class AsyncPortalsResourceWithStreamingResponse:
    def __init__(self, portals: AsyncPortalsResource) -> None:
        self._portals = portals

        self.create = async_to_streamed_response_wrapper(
            portals.create,
        )
        self.update = async_to_streamed_response_wrapper(
            portals.update,
        )
        self.list = async_to_streamed_response_wrapper(
            portals.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            portals.delete,
        )
        self.read = async_to_streamed_response_wrapper(
            portals.read,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SubscriptionComponent"]


class SubscriptionComponent(BaseModel):
    """A component value for a subscription."""

    default: Optional[float] = None
    """The default amount assigned."""

    display_name: Optional[str] = None
    """A human-readable version of the component name."""

    kind: Optional[Literal["enum", "sum", "usage"]] = None
    """The type of component value.

    "enum" for discrete values (including boolean on/off toggles where 0=off and
    1=on), "sum" for countable quantities, "usage" for metered billing components.
    """

    name: Optional[str] = None
    """The name of the component value."""

    price: Optional[float] = None
    """The unit price for the component value."""

    value: Optional[float] = None
    """The amount of the component value assigned."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["SensitivityGroupCreateParams", "Level"]


class SensitivityGroupCreateParams(TypedDict, total=False):
    account_id: Required[str]

    name: Required[str]

    description: Optional[str]

    levels: Iterable[Level]
    """Levels to create with the group. Mutually exclusive with `template_id`."""

    template_id: Optional[str]


class Level(TypedDict, total=False):
    name: Required[str]

    description: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .schema_http import SchemaHTTP

__all__ = ["DEXTestDeleteResponse"]


class DEXTestDeleteResponse(BaseModel):
    dex_tests: Optional[List[SchemaHTTP]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .applications import (
    ApplicationsResource,
    AsyncApplicationsResource,
    ApplicationsResourceWithRawResponse,
    AsyncApplicationsResourceWithRawResponse,
    ApplicationsResourceWithStreamingResponse,
    AsyncApplicationsResourceWithStreamingResponse,
)
from .auth_methods import (
    AuthMethodsResource,
    AsyncAuthMethodsResource,
    AuthMethodsResourceWithRawResponse,
    AsyncAuthMethodsResourceWithRawResponse,
    AuthMethodsResourceWithStreamingResponse,
    AsyncAuthMethodsResourceWithStreamingResponse,
)

__all__ = [
    "AuthMethodsResource",
    "AsyncAuthMethodsResource",
    "AuthMethodsResourceWithRawResponse",
    "AsyncAuthMethodsResourceWithRawResponse",
    "AuthMethodsResourceWithStreamingResponse",
    "AsyncAuthMethodsResourceWithStreamingResponse",
    "ApplicationsResource",
    "AsyncApplicationsResource",
    "ApplicationsResourceWithRawResponse",
    "AsyncApplicationsResourceWithRawResponse",
    "ApplicationsResourceWithStreamingResponse",
    "AsyncApplicationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .robot_get_params import RobotGetParams as RobotGetParams
from .robot_get_response import RobotGetResponse as RobotGetResponse
from .robot_bulk_get_params import RobotBulkGetParams as RobotBulkGetParams
from .robot_bulk_get_response import RobotBulkGetResponse as RobotBulkGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .findings import (
    FindingsResource,
    AsyncFindingsResource,
    FindingsResourceWithRawResponse,
    AsyncFindingsResourceWithRawResponse,
    FindingsResourceWithStreamingResponse,
    AsyncFindingsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .content_findings import (
    ContentFindingsResource,
    AsyncContentFindingsResource,
    ContentFindingsResourceWithRawResponse,
    AsyncContentFindingsResourceWithRawResponse,
    ContentFindingsResourceWithStreamingResponse,
    AsyncContentFindingsResourceWithStreamingResponse,
)

__all__ = ["DataSecurityResource", "AsyncDataSecurityResource"]


class DataSecurityResource(SyncAPIResource):
    @cached_property
    def content_findings(self) -> ContentFindingsResource:
        return ContentFindingsResource(self._client)

    @cached_property
    def findings(self) -> FindingsResource:
        return FindingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DataSecurityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DataSecurityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataSecurityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DataSecurityResourceWithStreamingResponse(self)


class AsyncDataSecurityResource(AsyncAPIResource):
    @cached_property
    def content_findings(self) -> AsyncContentFindingsResource:
        return AsyncContentFindingsResource(self._client)

    @cached_property
    def findings(self) -> AsyncFindingsResource:
        return AsyncFindingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDataSecurityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataSecurityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataSecurityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDataSecurityResourceWithStreamingResponse(self)


class DataSecurityResourceWithRawResponse:
    def __init__(self, data_security: DataSecurityResource) -> None:
        self._data_security = data_security

    @cached_property
    def content_findings(self) -> ContentFindingsResourceWithRawResponse:
        return ContentFindingsResourceWithRawResponse(self._data_security.content_findings)

    @cached_property
    def findings(self) -> FindingsResourceWithRawResponse:
        return FindingsResourceWithRawResponse(self._data_security.findings)


class AsyncDataSecurityResourceWithRawResponse:
    def __init__(self, data_security: AsyncDataSecurityResource) -> None:
        self._data_security = data_security

    @cached_property
    def content_findings(self) -> AsyncContentFindingsResourceWithRawResponse:
        return AsyncContentFindingsResourceWithRawResponse(self._data_security.content_findings)

    @cached_property
    def findings(self) -> AsyncFindingsResourceWithRawResponse:
        return AsyncFindingsResourceWithRawResponse(self._data_security.findings)


class DataSecurityResourceWithStreamingResponse:
    def __init__(self, data_security: DataSecurityResource) -> None:
        self._data_security = data_security

    @cached_property
    def content_findings(self) -> ContentFindingsResourceWithStreamingResponse:
        return ContentFindingsResourceWithStreamingResponse(self._data_security.content_findings)

    @cached_property
    def findings(self) -> FindingsResourceWithStreamingResponse:
        return FindingsResourceWithStreamingResponse(self._data_security.findings)


class AsyncDataSecurityResourceWithStreamingResponse:
    def __init__(self, data_security: AsyncDataSecurityResource) -> None:
        self._data_security = data_security

    @cached_property
    def content_findings(self) -> AsyncContentFindingsResourceWithStreamingResponse:
        return AsyncContentFindingsResourceWithStreamingResponse(self._data_security.content_findings)

    @cached_property
    def findings(self) -> AsyncFindingsResourceWithStreamingResponse:
        return AsyncFindingsResourceWithStreamingResponse(self._data_security.findings)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["LatestListResponse", "Item", "ItemE"]


class ItemE(BaseModel):
    """Event kind plus event-specific payload fields.

    Event kinds:
    - `Init`: Initialized process
    - `Leave`: Stopped process
    - `StartAttestation`: Started attestation
    - `FinishAttestationSuccess`: Finished attestation
    - `FinishAttestationFailure`: Failed attestation
    - `StartRotateCryptKey`: Started crypt key rotation
    - `FinishRotateCryptKeySuccess`: Finished crypt key rotation
    - `FinishRotateCryptKeyFailure`: Failed crypt key rotation
    - `StartRotatePki`: Started PKI rotation
    - `FinishRotatePkiSuccess`: Finished PKI rotation
    - `FinishRotatePkiFailure`: Failed PKI rotation
    - `StartUpgrade`: Started upgrade
    - `FinishUpgradeSuccess`: Finished upgrade
    - `FinishUpgradeFailure`: Failed upgrade
    - `Reconcile`: Reconciled
    - `ConfigureCloudflaredTunnel`: Configured Cloudflared tunnel
    - `RekeyInstallBoth`: Installed initial inbound and outbound keys
    - `RekeyStart`: Installed new inbound key, kept old outbound
    - `RekeyRestart`: Restarted in-progress rekey with newer key material
    - `RekeyAdvance`: Confirmed traffic on new inbound key, swapped outbound to new
    - `RekeyComplete`: Deleted old keys
    - `RekeyReset`: Deleted all keys after receiving an unexpected key
    """

    k: Literal[
        "Init",
        "Leave",
        "StartAttestation",
        "FinishAttestationSuccess",
        "FinishAttestationFailure",
        "StartRotateCryptKey",
        "FinishRotateCryptKeySuccess",
        "FinishRotateCryptKeyFailure",
        "StartRotatePki",
        "FinishRotatePkiSuccess",
        "FinishRotatePkiFailure",
        "StartUpgrade",
        "FinishUpgradeSuccess",
        "FinishUpgradeFailure",
        "Reconcile",
        "ConfigureCloudflaredTunnel",
        "RekeyInstallBoth",
        "RekeyStart",
        "RekeyRestart",
        "RekeyAdvance",
        "RekeyComplete",
        "RekeyReset",
    ]
    """Event kind"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Item(BaseModel):
    """Recorded Event"""

    e: ItemE
    """Event kind plus event-specific payload fields.

    Event kinds:

    - `Init`: Initialized process
    - `Leave`: Stopped process
    - `StartAttestation`: Started attestation
    - `FinishAttestationSuccess`: Finished attestation
    - `FinishAttestationFailure`: Failed attestation
    - `StartRotateCryptKey`: Started crypt key rotation
    - `FinishRotateCryptKeySuccess`: Finished crypt key rotation
    - `FinishRotateCryptKeyFailure`: Failed crypt key rotation
    - `StartRotatePki`: Started PKI rotation
    - `FinishRotatePkiSuccess`: Finished PKI rotation
    - `FinishRotatePkiFailure`: Failed PKI rotation
    - `StartUpgrade`: Started upgrade
    - `FinishUpgradeSuccess`: Finished upgrade
    - `FinishUpgradeFailure`: Failed upgrade
    - `Reconcile`: Reconciled
    - `ConfigureCloudflaredTunnel`: Configured Cloudflared tunnel
    - `RekeyInstallBoth`: Installed initial inbound and outbound keys
    - `RekeyStart`: Installed new inbound key, kept old outbound
    - `RekeyRestart`: Restarted in-progress rekey with newer key material
    - `RekeyAdvance`: Confirmed traffic on new inbound key, swapped outbound to new
    - `RekeyComplete`: Deleted old keys
    - `RekeyReset`: Deleted all keys after receiving an unexpected key
    """

    n: float
    """Sequence number, used to order events with the same timestamp"""

    t: float
    """Time the Event was recorded (seconds since the Unix epoch)"""

    v: Optional[str] = None
    """Version"""


class LatestListResponse(BaseModel):
    count: float

    items: List[Item]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .moq import (
    MoQResource,
    AsyncMoQResource,
    MoQResourceWithRawResponse,
    AsyncMoQResourceWithRawResponse,
    MoQResourceWithStreamingResponse,
    AsyncMoQResourceWithStreamingResponse,
)
from .relays import (
    RelaysResource,
    AsyncRelaysResource,
    RelaysResourceWithRawResponse,
    AsyncRelaysResourceWithRawResponse,
    RelaysResourceWithStreamingResponse,
    AsyncRelaysResourceWithStreamingResponse,
)

__all__ = [
    "RelaysResource",
    "AsyncRelaysResource",
    "RelaysResourceWithRawResponse",
    "AsyncRelaysResourceWithRawResponse",
    "RelaysResourceWithStreamingResponse",
    "AsyncRelaysResourceWithStreamingResponse",
    "MoQResource",
    "AsyncMoQResource",
    "MoQResourceWithRawResponse",
    "AsyncMoQResourceWithRawResponse",
    "MoQResourceWithStreamingResponse",
    "AsyncMoQResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EventListParams"]


class EventListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Maximum number of results per page. Must be a multiple of 5."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["DMARCReportEditParams"]


class DMARCReportEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    enabled: Optional[bool]
    """Enable or disable DMARC reports for this zone"""

    skip_wizard: Optional[bool]
    """Skip the DMARC setup wizard"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SnapshotUpdateParams"]


class SnapshotUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    source: str
    """The file to upload.

    Either a plain STIX2/CRDF body or a gzipped one (recognised by `0x1f 0x8b` magic
    bytes or a `.gz` filename suffix).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ......_models import BaseModel

__all__ = ["JobCreateResponse", "Created", "CreatedParameters", "Failed"]


class CreatedParameters(BaseModel):
    """Parameters for a webhook job."""

    finding_instance_id: str
    """ID of the finding instance."""


class Created(BaseModel):
    """Information about a webhook job."""

    id: str
    """Unique identifier for the webhook job."""

    asset_data: Dict[str, object]
    """Asset data associated with this webhook job."""

    created_at: datetime
    """When the webhook job was created."""

    integration_id: str
    """ID of the integration."""

    last_updated_at: datetime
    """When the webhook job was last updated."""

    parameters: CreatedParameters
    """Parameters for a webhook job."""

    status: Literal["pending", "processing", "completed", "failed"]
    """Status of a webhook job."""

    triggered_by_actor: Literal["user", "account_token"]
    """Type of actor that triggered the webhook job."""

    triggered_by_id: str
    """ID of the actor that triggered the job."""

    webhook_id: str
    """ID of the webhook configuration."""

    failure_details: Optional[Dict[str, object]] = None
    """Additional details about the failure."""

    failure_reason: Optional[
        Literal["Permission Denied", "Integration Unavailable", "Service Temporarily Unavailable", "System Error"]
    ] = None
    """Reason for webhook job failure."""


class Failed(BaseModel):
    """Information about a failed webhook job creation."""

    error: str
    """Error message describing the failure."""

    finding_instance_id: str
    """ID of the finding instance that failed to create a webhook job."""

    webhook_id: str
    """ID of the webhook configuration."""


class JobCreateResponse(BaseModel):
    created: List[Created]
    """Successfully created webhook jobs."""

    failed: List[Failed]
    """Failed webhook job creation attempts."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EmailRoutingUnlockParams"]


class EmailRoutingUnlockParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    name: str
    """Domain of your zone."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["RelayListParams"]


class RelayListParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account identifier."""

    asc: bool
    """Sort order by `created`.

    When true, results are returned oldest-first (ascending); otherwise newest-first
    (descending, the default).
    """

    created_after: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Cursor for pagination.

    Returns relays created strictly after this RFC 3339 timestamp (typically the
    `created` value of the last item on the current page, to fetch the next page).
    """

    created_before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Cursor for pagination.

    Returns relays created strictly before this RFC 3339 timestamp (typically the
    `created` value of the first item on the current page, to fetch the previous
    page).
    """

    per_page: int
    """Maximum number of relays to return per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["UserSchemaListParams"]


class UserSchemaListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    omit_source: bool
    """Omit the source-files of schemas and only retrieve their meta-data."""

    page: int
    """Page number of paginated results."""

    per_page: int
    """Maximum number of results per page."""

    validation_enabled: bool
    """Flag whether schema is enabled for validation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["IndicatorListParams", "Search", "TagSearch"]


class IndicatorListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    cache: Literal["from-graph"]
    """Cache strategy.

    'from-graph' serves results from the graph-node KV cache when all requested
    UUIDs are cached; falls back to normal path on partial/zero hit.
    """

    created_after: Annotated[Union[str, datetime], PropertyInfo(alias="createdAfter", format="iso8601")]
    """Filter indicators created on or after this date.

    Must use ISO 8601 format (e.g., '2024-01-15T00:00:00Z').
    """

    created_before: Annotated[Union[str, datetime], PropertyInfo(alias="createdBefore", format="iso8601")]
    """Filter indicators created on or before this date.

    Must use ISO 8601 format (e.g., '2024-12-31T23:59:59Z').
    """

    dataset_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="datasetIds")]
    """
    Dataset IDs to query indicators from (array of UUIDs), or special value 'all' or
    '\\**' to query all datasets. If not provided, uses the default dataset.
    """

    format: Literal["json", "stix2", "taxii"]
    """Output format for indicator data.

    'json' returns the default format, 'stix2' returns STIX 2.1 Indicator SDOs,
    'taxii' returns a TAXII 2.1 Envelope with Content-Type
    application/taxii+json;version=2.1.
    """

    include_tags: Annotated[bool, PropertyInfo(alias="includeTags")]
    """Whether to include full tag details for each indicator. Defaults to true."""

    include_total_count: Annotated[bool, PropertyInfo(alias="includeTotalCount")]
    """Whether to compute accurate total count via COUNT(\\**).

    Defaults to false for performance. When false, total_count is an approximation.
    """

    indicator_type: Annotated[str, PropertyInfo(alias="indicatorType")]

    name: str
    """Filter indicators by value using substring match (LIKE).

    Legacy alternative to structured search.
    """

    page: float

    page_size: Annotated[float, PropertyInfo(alias="pageSize")]

    related_events: Annotated[SequenceNotStr[str], PropertyInfo(alias="relatedEvents")]
    """Filter by related event IDs"""

    related_events_limit: Annotated[float, PropertyInfo(alias="relatedEventsLimit")]
    """Limit the number of related events returned per indicator.

    Default: 2. Set to 0 for none, -1 for all events.
    """

    search: Iterable[Search]
    """Structured search as a JSON array of {field, op, value} objects.

    Searchable fields: value, indicatorType, uuid. Supports operators: equals, not,
    contains, startsWith, endsWith, gt, lt, gte, lte, like, in, find. Use the 'in'
    operator with an array value to bulk-check up to 100 indicators in a single
    request, e.g.
    search=[{"field":"value","op":"in","value":["evil.com","bad.org"]}]. Multiple
    conditions are AND'd together. Max 10 conditions per request.
    """

    source: Literal["do", "r2catalog"]
    """Read backend.

    'do' (default) reads Durable Object storage. 'r2catalog' reads R2 Data Catalog
    (admin-only, experimental; supports a subset of search fields).
    """

    tags: SequenceNotStr[str]
    """Filter by tag values or UUIDs.

    Indicators must have at least one of the specified tags (OR logic). Supports
    both tag UUID and tag value.
    """

    tag_search: Annotated[Iterable[TagSearch], PropertyInfo(alias="tagSearch")]
    """Structured tag-metadata filter as a JSON array of {field, op, value} objects.

    Operates against the per-dataset IndicatorTag mirror so you can find indicators
    by tag attributes (origin country, motive, sophistication, priority, etc.)
    without a separate Tags lookup. Common dashboard usage: drill from a country
    into indicators, e.g.
    tagSearch=[{"field":"originCountryISO","op":"in","value":["IR","CN"]}]. Country
    values may be passed as alpha-2, alpha-3, name, or alias (e.g. "iran").
    Operators: equals, not, gt/gte/lt/lte (numeric only),
    contains/like/find/startsWith/endsWith (string only), in. AND-joined across
    entries; combined with `tags`, a matching tag must satisfy both. Max 10 entries
    per request, max 100 values per 'in'. Performance notes: `originCountryISO` uses
    its B-tree index for equals/not/in. `priority` uses its B-tree index for numeric
    comparisons. Other string columns (`actorCategory`, `motive`, etc.) are
    case-insensitive and unindexed; current catalog size makes this a non-issue.
    `endsWith` and `aliasGroupNames` contains/like are leading-wildcard scans and
    slow on large result sets. `aliasGroupNames` matches on the JSON-encoded text,
    so substrings can cross alias boundaries ("apt28" also matches "apt280" when
    both appear in the same tag's alias list).
    """


class Search(TypedDict, total=False):
    field: Required[Literal["value", "indicatorType", "uuid"]]
    """The indicator field to search on. Allowed: value, indicatorType, uuid."""

    op: Required[
        Literal["equals", "not", "gt", "gte", "lt", "lte", "like", "contains", "startsWith", "endsWith", "in", "find"]
    ]
    """Search operator.

    Use 'in' for bulk lookup of up to 100 values at once, e.g. {field:'value',
    op:'in', value:['evil.com','bad.org']}.
    """

    value: Required[Union[str, SequenceNotStr[str]]]
    """Search value.

    String for most operators. Array of strings for 'in' operator (max 100 items).
    """


class TagSearch(TypedDict, total=False):
    field: Required[
        Literal[
            "value",
            "categoryId",
            "actorCategory",
            "aliasGroupNames",
            "attributionConfidence",
            "attributionOrganization",
            "motive",
            "opsecLevel",
            "originCountryISO",
            "sophisticationLevel",
            "priority",
            "analyticPriority",
        ]
    ]
    """Tag mirror field to filter on.

    Allowed: value, categoryId, actorCategory, aliasGroupNames,
    attributionConfidence, attributionOrganization, motive, opsecLevel,
    originCountryISO, sophisticationLevel, priority, analyticPriority. Filters
    operate against the per-dataset IndicatorTag mirror (which is kept in sync with
    the Tags SoT by the tag-propagation workflow).
    """

    op: Required[
        Literal["equals", "not", "gt", "gte", "lt", "lte", "like", "contains", "startsWith", "endsWith", "in", "find"]
    ]
    """Search operator.

    Use 'in' for bulk OR within a single field, e.g. {field:"originCountryISO",
    op:"in", value:["IR","CN"]}.
    """

    value: Union[str, float, SequenceNotStr[Union[str, float]]]
    """Search value.

    String or number for most operators. Array for 'in' (max 100 items). Country
    values may be passed as alpha-2, alpha-3, name, or common alias (e.g. "iran",
    "IR", "IRN") and are normalized to alpha-2 server-side.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["FindingIgnoreParams"]


class FindingIgnoreParams(TypedDict, total=False):
    account_id: Required[str]

    checks: Required[SequenceNotStr[str]]
    """A list of finding IDs to pass along."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["TenantMembership"]


class TenantMembership(BaseModel):
    user_email: str

    user_name: str

    user_tag: str
